@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <stdio.h>
 
 #include "common/avl.h"
@@ -59,9 +63,12 @@
 
 static void _apply_metric(struct nhdp_domain *domain, const char *metric_name);
 static void _remove_metric(struct nhdp_domain *);
-static void _apply_mpr(struct nhdp_domain *domain, const char *mpr_name);
+static void _apply_mpr(struct nhdp_domain *domain,
+    const char *mpr_name, uint8_t willingness);
 static void _remove_mpr(struct nhdp_domain *);
 
+static void _cb_update_everyone_mpr(void);
+
 static void _recalculate_neighbor_metric(struct nhdp_domain *domain,
         struct nhdp_neighbor *neigh);
 static const char *_link_to_string(struct nhdp_metric_str *, uint32_t);
@@ -69,7 +76,7 @@ static const char *_path_to_string(struct nhdp_metric_str *, uint32_t, uint8_t);
 static const char *_int_to_string(struct nhdp_metric_str *, struct nhdp_link *);
 
 /* domain class */
-struct oonf_class _domain_class = {
+static struct oonf_class _domain_class = {
   .name = NHDP_CLASS_DOMAIN,
   .size = sizeof(struct nhdp_domain),
 };
@@ -91,12 +98,13 @@ static struct nhdp_domain_metric _no_metric = {
 };
 
 /* default MPR handler (no MPR handling) */
-static struct nhdp_domain_mpr _no_mprs = {
+static struct nhdp_domain_mpr _everyone_mprs = {
   .name = "Everyone MPR",
 
   .mpr_start = true,
   .mprs_start = true,
-  .willingness = RFC7181_WILLINGNESS_DEFAULT,
+
+  .update_mpr = _cb_update_everyone_mpr,
 };
 
 /* non-default routing domains registered to NHDP */
@@ -106,12 +114,11 @@ static struct list_entity _domain_listener_list;
 static size_t _domain_counter = 0;
 
 /* tree of known routing metrics/mpr-algorithms */
-struct avl_tree nhdp_domain_metrics;
-struct avl_tree nhdp_domain_mprs;
+static struct avl_tree _domain_metrics;
+static struct avl_tree _domain_mprs;
 
-/* flooding MPR handler registered to NHDP */
-static struct nhdp_domain_mpr *_flooding_mpr = &_no_mprs;
-uint8_t _flooding_ext = 0;
+/* flooding domain */
+static struct nhdp_domain _flooding_domain;
 
 /* NHDP RFC5444 protocol */
 static struct oonf_rfc5444_protocol *_protocol;
@@ -131,8 +138,15 @@ nhdp_domain_init(struct oonf_rfc5444_protocol *p) {
   list_init_head(&_domain_list);
   list_init_head(&_domain_listener_list);
 
-  avl_init(&nhdp_domain_metrics, avl_comp_strcasecmp, false);
-  avl_init(&nhdp_domain_mprs, avl_comp_strcasecmp, false);
+  avl_init(&_domain_metrics, avl_comp_strcasecmp, false);
+  avl_init(&_domain_mprs, avl_comp_strcasecmp, false);
+
+  /* initialize flooding domain */
+  _flooding_domain.metric = &_no_metric;
+  _flooding_domain.mpr = &_everyone_mprs;
+
+  _flooding_domain.mpr->_refcount++;
+  _flooding_domain.metric->_refcount++;
 }
 
 /**
@@ -170,14 +184,6 @@ nhdp_domain_get_count(void) {
   return _domain_counter;
 }
 
-/**
- * @return current flooding mpr
- */
-struct nhdp_domain_mpr *
-nhdp_domain_get_flooding_mpr(void) {
-  return _flooding_mpr;
-}
-
 /**
  * Add a new metric handler to nhdp
  * @param metric pointer to NHDP link metric
@@ -215,7 +221,7 @@ nhdp_domain_metric_add(struct nhdp_domain_metric *metric) {
   }
 
   /* hook into tree */
-  return avl_insert(&nhdp_domain_metrics, &metric->_node);
+  return avl_insert(&_domain_metrics, &metric->_node);
 }
 
 /**
@@ -233,7 +239,7 @@ nhdp_domain_metric_remove(struct nhdp_domain_metric *metric) {
     }
   }
 
-  avl_remove(&nhdp_domain_metrics, &metric->_node);
+  avl_remove(&_domain_metrics, &metric->_node);
 }
 
 /**
@@ -248,15 +254,19 @@ nhdp_domain_mpr_add(struct nhdp_domain_mpr *mpr) {
   /* initialize key */
   mpr->_node.key = mpr->name;
 
-  if (avl_insert(&nhdp_domain_mprs, &mpr->_node)) {
+  if (avl_insert(&_domain_mprs, &mpr->_node)) {
     return -1;
   }
 
   list_for_each_element(&_domain_list, domain, _node) {
-    if (domain->mpr == &_no_mprs) {
-      _apply_mpr(domain, domain->mpr_name);
+    if (domain->mpr == &_everyone_mprs) {
+      _apply_mpr(domain, domain->mpr_name, domain->local_willingness);
     }
   }
+  if (_flooding_domain.mpr == &_everyone_mprs) {
+    _apply_mpr(&_flooding_domain,
+        _flooding_domain.mpr_name, _flooding_domain.local_willingness);
+  }
   return 0;
 }
 
@@ -275,7 +285,7 @@ nhdp_domain_mpr_remove(struct nhdp_domain_mpr *mpr) {
     }
   }
 
-  avl_remove(&nhdp_domain_mprs, &mpr->_node);
+  avl_remove(&_domain_mprs, &mpr->_node);
 }
 
 /**
@@ -372,9 +382,9 @@ nhdp_domain_init_neighbor(struct nhdp_neighbor *neigh) {
   int i;
 
   /* initialize flooding MPR settings */
-  neigh->flooding_willingness = _flooding_mpr->willingness;
-  neigh->local_is_flooding_mpr = _flooding_mpr->mprs_start;
-  neigh->neigh_is_flooding_mpr = _flooding_mpr->mpr_start;
+  neigh->flooding_willingness = RFC7181_WILLINGNESS_NEVER;
+  neigh->local_is_flooding_mpr = _flooding_domain.mpr->mprs_start;
+  neigh->neigh_is_flooding_mpr = _flooding_domain.mpr->mpr_start;
 
   for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
     neigh->_domaindata[i].metric.in = RFC7181_METRIC_MAX;
@@ -390,7 +400,7 @@ nhdp_domain_init_neighbor(struct nhdp_neighbor *neigh) {
 
     data->best_link = NULL;
 
-    data->willingness = domain->mpr->willingness;
+    data->willingness = RFC7181_WILLINGNESS_NEVER;
     data->local_is_mpr = domain->mpr->mprs_start;
     data->neigh_is_mpr = domain->mpr->mpr_start;
   }
@@ -424,7 +434,7 @@ nhdp_domain_process_metric_linktlv(struct nhdp_domain *domain,
  * Process an in linkmetric tlv for a nhdp twohop neighbor
  * @param domain pointer to NHDP domain
  * @param l2hop pointer to nhdp twohop neighbor
- * @param tlvvalue value of metric tlv
+ * @param value value of metric tlv
  */
 void
 nhdp_domain_process_metric_2hoptlv(struct nhdp_domain *domain,
@@ -498,6 +508,7 @@ nhdp_domain_neighbor_changed(struct nhdp_neighbor *neigh) {
   struct nhdp_domain_listener *listener;
   struct nhdp_domain *domain;
   
+
   list_for_each_element(&_domain_list, domain, _node) {
     _recalculate_neighbor_metric(domain, neigh);
 
@@ -574,8 +585,8 @@ nhdp_domain_process_mprtypes_tlv(
  * Process an in MPR tlv for a NHDP link
  * @param mprtypes list of extenstions for MPR
  * @param mprtypes_size length of mprtypes array
- * @param lnk NHDP link
- * @param tlv MPR tlv
+ * @param neigh NHDP neighbor
+ * @param tlv MPR tlv context
  */
 void
 nhdp_domain_process_mpr_tlv(uint8_t *mprtypes, size_t mprtypes_size,
@@ -622,22 +633,23 @@ nhdp_domain_process_mpr_tlv(uint8_t *mprtypes, size_t mprtypes_size,
 
 /**
  * Process an in Willingness tlv and put values into
- * temporary storage in MPR handler object
+ * temporary storage in MPR handler object. Call
+ * nhdp_domain_store_willingness to permanently store them later.
  * @param mprtypes list of extenstions for MPR
  * @param mprtypes_size length of mprtypes array
- * @param tlvvalue willingness array to parse
- * @param tlvsize length of willingness array
+ * @param tlv Willingness tlv context
  */
 void
-nhdp_domain_process_willingness_tlv(uint8_t *mprtypes, size_t mprtypes_size,
+nhdp_domain_process_willingness_tlv(
+    uint8_t *mprtypes, size_t mprtypes_size,
     struct rfc5444_reader_tlvblock_entry *tlv) {
   struct nhdp_domain *domain;
   size_t idx, i;
   uint8_t value;
 
-  _flooding_mpr->willingness = RFC7181_WILLINGNESS_NEVER;
+  _flooding_domain._tmp_willingness = RFC7181_WILLINGNESS_NEVER;
   list_for_each_element(&_domain_list, domain, _node) {
-    domain->mpr->willingness = RFC7181_WILLINGNESS_NEVER;
+    domain->_tmp_willingness= RFC7181_WILLINGNESS_NEVER;
   }
 
   if (!tlv) {
@@ -645,9 +657,10 @@ nhdp_domain_process_willingness_tlv(uint8_t *mprtypes, size_t mprtypes_size,
   }
 
   /* copy flooding willingness */
-  _flooding_mpr->willingness = tlv->single_value[0] & RFC7181_WILLINGNESS_MASK;
+  _flooding_domain._tmp_willingness
+    = tlv->single_value[0] & RFC7181_WILLINGNESS_MASK;
   OONF_DEBUG(LOG_NHDP_R, "Received flooding willingness: %u",
-      _flooding_mpr->willingness);
+      _flooding_domain._tmp_willingness);
 
   for (i=0; i<mprtypes_size; i++) {
     domain = nhdp_domain_get_by_ext(mprtypes[i]);
@@ -668,10 +681,32 @@ nhdp_domain_process_willingness_tlv(uint8_t *mprtypes, size_t mprtypes_size,
       value &= RFC7181_WILLINGNESS_MASK;
     }
 
-    domain->mpr->willingness = value;
+    domain->_tmp_willingness = value;
 
     OONF_DEBUG(LOG_NHDP_R, "Received routing willingness for domain %u: %u",
-        domain->ext, _flooding_mpr->willingness);
+        domain->ext, domain->_tmp_willingness);
+  }
+}
+
+/**
+ * Stores the willingness data processed by
+ * nhdp_domain_process_willingness_tlv() into a neighbor object
+ * @param neigh NHDP neighbor
+ */
+void
+nhdp_domain_store_willingness(struct nhdp_neighbor *neigh) {
+  struct nhdp_neighbor_domaindata *neighdata;
+  struct nhdp_domain *domain;
+
+  neigh->flooding_willingness = _flooding_domain._tmp_willingness;
+  OONF_DEBUG(LOG_NHDP_R, "Set flooding willingness: %u",
+      neigh->flooding_willingness);
+
+  list_for_each_element(&_domain_list, domain, _node) {
+    neighdata = nhdp_domain_get_neighbordata(domain, neigh);
+    neighdata->willingness = domain->_tmp_willingness;
+    OONF_DEBUG(LOG_NHDP_R, "Set routing willingness for domain %u: %u",
+        domain->ext, neighdata->willingness);
   }
 }
 
@@ -760,9 +795,9 @@ nhdp_domain_encode_willingness_tlvvalue(uint8_t *tlvvalue, size_t tlvsize) {
   len = 0;
 
   /* set flooding willingness */
-  tlvvalue[0] = _flooding_mpr->willingness;
+  tlvvalue[0] = _flooding_domain.local_willingness;
   OONF_DEBUG(LOG_NHDP_W, "Set flooding willingness: %u",
-      _flooding_mpr->willingness);
+      _flooding_domain.local_willingness);
 
   /* set routing willingness */
   list_for_each_element(&_domain_list, domain, _node) {
@@ -774,15 +809,16 @@ nhdp_domain_encode_willingness_tlvvalue(uint8_t *tlvvalue, size_t tlvsize) {
       len = idx + 1;
     }
 
-    value = domain->mpr->willingness & RFC7181_WILLINGNESS_MASK;
+    value = domain->local_willingness & RFC7181_WILLINGNESS_MASK;
 
-    OONF_DEBUG(LOG_NHDP_W, "Set routing willingness for domain %u: %u"
+    if ((domain->index & 1) == 0) {
+      value <<= RFC7181_WILLINGNESS_SHIFT;
+    }
+
+    OONF_DEBUG(LOG_NHDP_W, "Set routing willingness for domain %u: %x"
         " (%"PRINTF_SIZE_T_SPECIFIER")",
         domain->ext, value, idx);
 
-    if ((idx & 1) == 0) {
-      value <<= RFC7181_WILLINGNESS_SHIFT;
-    }
     tlvvalue[idx] |= value;
   }
 
@@ -791,25 +827,18 @@ nhdp_domain_encode_willingness_tlvvalue(uint8_t *tlvvalue, size_t tlvsize) {
 
 /**
  * Sets a new flodding MPR algorithm
- * @param mpr pointer to flooding MPR handler
- * @param ext TLV extension to transport flooding MPR settings
+ * @param mpr_name name of MPR algorithm
+ * @param willingness of MPR algorithm
  */
 void
-nhdp_domain_set_flooding_mpr(struct nhdp_domain_mpr *mpr, uint8_t ext) {
-  if (mpr == NULL) {
-    _flooding_mpr = &_no_mprs;
-    _flooding_ext = 0;
-  }
-  else {
-    _flooding_mpr = mpr;
-    _flooding_ext = ext;
-  }
+nhdp_domain_set_flooding_mpr(const char *mpr_name, uint8_t willingness) {
+  _apply_mpr(&_flooding_domain, mpr_name, willingness);
 }
 
 /**
  * Sets the incoming metric of a link. This is the only function external
  * code should use to commit the calculated metric values to the nhdp db.
- * @param domain NHDP domain
+ * @param metric NHDP domain metric
  * @param lnk NHDP link
  * @param metric_in incoming metric value for NHDP link
  * @return true if metric changed, false otherwise
@@ -819,31 +848,45 @@ nhdp_domain_set_incoming_metric(struct nhdp_domain_metric *metric,
     struct nhdp_link *lnk, uint32_t metric_in) {
   struct nhdp_link_domaindata *linkdata;
   struct nhdp_domain *domain;
-  uint32_t old_metric;
   bool changed;
 
   changed = false;
   list_for_each_element(&_domain_list, domain, _node) {
     if (domain->metric == metric) {
       linkdata = nhdp_domain_get_linkdata(domain, lnk);
-      old_metric = linkdata->metric.in;
+      changed |= (linkdata->metric.in != metric_in);
       linkdata->metric.in = metric_in;
-      changed |= (old_metric != metric_in);
     }
   }
   return changed;
 }
 
+/**
+ * get list of nhdp domains
+ * @return domain list
+ */
 struct list_entity *
 nhdp_domain_get_list(void) {
   return &_domain_list;
 }
 
+/**
+ * get list of nhdp domain listeners for metric/mpr changes
+ * @return listener list
+ */
 struct list_entity *
 nhdp_domain_get_listener_list(void) {
   return &_domain_listener_list;
 }
 
+/**
+ * get current NHDP flooding domain
+ * @return flooding domain
+ */
+const struct nhdp_domain *
+nhdp_domain_get_flooding(void) {
+  return &_flooding_domain;
+}
 
 /**
  * Recalculate the 'best link/metric' values of a neighbor
@@ -926,7 +969,10 @@ nhdp_domain_add(uint8_t ext) {
   domain->ext = ext;
   domain->index = _domain_counter++;
   domain->metric = &_no_metric;
-  domain->mpr = &_no_mprs;
+  domain->mpr = &_everyone_mprs;
+
+  domain->mpr->_refcount++;
+  domain->metric->_refcount++;
 
   /* initialize metric TLVs */
   for (i=0; i<4; i++) {
@@ -955,11 +1001,13 @@ nhdp_domain_add(uint8_t ext) {
  *   might be CFG_DOMAIN_NO_MPR (every node is MPR)
  *   or CFG_DOMAIN_ANY_MPR (for a MPR the NHDP core should
  *   choose).
+ * @param willingness routing willingness for domain
  * @return pointer to configured domain, NULL, if out of memory or
  *   maximum number of domains has been reached.
  */
 struct nhdp_domain *
-nhdp_domain_configure(uint8_t ext, const char *metric_name, const char *mpr_name) {
+nhdp_domain_configure(uint8_t ext, const char *metric_name,
+    const char *mpr_name, uint8_t willingness) {
   struct nhdp_domain *domain;
 
   domain = nhdp_domain_add(ext);
@@ -967,8 +1015,13 @@ nhdp_domain_configure(uint8_t ext, const char *metric_name, const char *mpr_name
     return NULL;
   }
 
+  OONF_DEBUG(LOG_NHDP, "Configure domain %u to metric=%s",
+      domain->index, metric_name);
   _apply_metric(domain, metric_name);
-  _apply_mpr(domain, mpr_name);
+
+  OONF_DEBUG(LOG_NHDP, "Configure domain %u to mpr=%s, willingness=%u",
+      domain->index, mpr_name, willingness);
+  _apply_mpr(domain, mpr_name, willingness);
 
   oonf_class_event(&_domain_class, domain, OONF_OBJECT_CHANGED);
 
@@ -988,30 +1041,32 @@ _apply_metric(struct nhdp_domain *domain, const char *metric_name) {
   struct nhdp_domain_metric *metric;
 
   /* check if we have to remove the old metric first */
-  if (strcasecmp(domain->metric_name, metric_name) != 0) {
-    if (domain->metric != &_no_metric) {
-      _remove_metric(domain);
-      strscpy(domain->metric_name, CFG_DOMAIN_NO_METRIC, sizeof(domain->metric_name));
-    }
-  }
-
-  /* Handle wildcard metric name first */
-  if (strcasecmp(metric_name, CFG_DOMAIN_ANY_METRIC) == 0
-      && !avl_is_empty(&nhdp_domain_metrics)) {
-    metric_name = avl_first_element(&nhdp_domain_metrics, metric, _node)->name;
-  }
-
-  /* copy new metric name */
-  strscpy(domain->metric_name, metric_name, sizeof(domain->metric_name));
-
-  /* look for metric implementation */
-  metric = avl_find_element(&nhdp_domain_metrics, metric_name, metric, _node);
-  if (metric == NULL) {
-    domain->metric = &_no_metric;
+  if (strcasecmp(domain->metric_name, metric_name) == 0) {
+    /* nothing to do, we already have the right metric */
     return;
   }
 
+  if (domain->metric != &_no_metric) {
+    _remove_metric(domain);
+  }
+
+  /* Handle wildcard metric name first */
+  if (strcasecmp(metric_name, CFG_DOMAIN_ANY_METRIC_MPR) == 0
+      && !avl_is_empty(&_domain_metrics)) {
+    metric_name = avl_first_element(&_domain_metrics, metric, _node)->name;
+  }
+
+  /* look for metric implementation */
+  metric = avl_find_element(&_domain_metrics, metric_name, metric, _node);
+  if (metric == NULL) {
+    metric = &_no_metric;
+  }
+
+  /* copy new metric name */
+  strscpy(domain->metric_name, metric->name, sizeof(domain->metric_name));
+
   /* link domain and metric */
+  domain->metric->_refcount--;
   domain->metric = metric;
 
   /* activate metric */
@@ -1028,13 +1083,12 @@ _apply_metric(struct nhdp_domain *domain, const char *metric_name) {
 static void
 _remove_metric(struct nhdp_domain *domain) {
   domain->metric->_refcount--;
-  if (!domain->metric->_refcount) {
-    if (domain->metric->disable) {
-      domain->metric->disable();
-    }
+  if (!domain->metric->_refcount && domain->metric->disable) {
+    domain->metric->disable();
   }
-  strscpy(domain->metric_name, CFG_DOMAIN_NO_METRIC, sizeof(domain->metric_name));
+  strscpy(domain->metric_name, CFG_DOMAIN_NO_METRIC_MPR, sizeof(domain->metric_name));
   domain->metric = &_no_metric;
+  domain->metric->_refcount++;
 }
 
 /**
@@ -1044,38 +1098,48 @@ _remove_metric(struct nhdp_domain *domain) {
  *   might be CFG_DOMAIN_NO_MPR (every node is MPR)
  *   or CFG_DOMAIN_ANY_MPR (for a MPR the NHDP core should
  *   choose).
+ * @param willingness routing willingness for domain
  */
 static void
-_apply_mpr(struct nhdp_domain *domain, const char *mpr_name) {
+_apply_mpr(struct nhdp_domain *domain, const char *mpr_name, uint8_t willingness) {
   struct nhdp_domain_mpr *mpr;
 
+  domain->local_willingness = willingness;
+
   /* check if we have to remove the old mpr first */
-  if (strcasecmp(domain->mpr_name, mpr_name) != 0) {
-    if (domain->mpr != &_no_mprs) {
-      _remove_mpr(domain);
-      strscpy(domain->mpr_name, CFG_DOMAIN_NO_MPR, sizeof(domain->mpr_name));
-    }
+  if (strcasecmp(domain->mpr_name, mpr_name) == 0) {
+    /* nothing else to do, we already have the right MPR */
+    return;
+  }
+  if (domain->mpr != &_everyone_mprs) {
+    /* replace old MPR algorithm with "everyone MPR" */
+    _remove_mpr(domain);
   }
 
   /* Handle wildcard mpr name first */
-  if (strcasecmp(mpr_name, CFG_DOMAIN_ANY_METRIC) == 0
-      && !avl_is_empty(&nhdp_domain_mprs)) {
-    mpr_name = avl_first_element(&nhdp_domain_mprs, mpr, _node)->name;
+  if (strcasecmp(mpr_name, CFG_DOMAIN_ANY_METRIC_MPR) == 0
+      && !avl_is_empty(&_domain_mprs)) {
+    mpr_name = avl_first_element(&_domain_mprs, mpr, _node)->name;
+  }
+
+  /* look for mpr implementation */
+  mpr = avl_find_element(&_domain_mprs, mpr_name, mpr, _node);
+  if (mpr == NULL) {
+    mpr = &_everyone_mprs;
   }
 
   /* copy new metric name */
-  strscpy(domain->mpr_name, mpr_name, sizeof(domain->mpr_name));
-
-  /* look for mpr implementation */
-  mpr = avl_find_element(&nhdp_domain_mprs, mpr_name, mpr, _node);
-  if (mpr == NULL) {
-    domain->mpr = &_no_mprs;
-    return;
-  }
+  strscpy(domain->mpr_name, mpr->name, sizeof(domain->mpr_name));
 
   /* link domain and mpr */
+  domain->mpr->_refcount--;
   domain->mpr = mpr;
-  mpr->domain = domain;
+
+  /* activate mpr */
+  if (mpr->_refcount == 0 && mpr->enable) {
+    mpr->enable();
+  }
+  mpr->_refcount++;
 }
 
 /**
@@ -1084,9 +1148,35 @@ _apply_mpr(struct nhdp_domain *domain, const char *mpr_name) {
  */
 static void
 _remove_mpr(struct nhdp_domain *domain) {
-  strscpy(domain->mpr_name, CFG_DOMAIN_NO_MPR, sizeof(domain->mpr_name));
-  domain->mpr->domain = NULL;
-  domain->mpr = &_no_mprs;
+  domain->mpr->_refcount--;
+  if (!domain->mpr->_refcount && domain->mpr->disable) {
+    domain->mpr->disable();
+  }
+  strscpy(domain->mpr_name, CFG_DOMAIN_NO_METRIC_MPR, sizeof(domain->mpr_name));
+  domain->mpr = &_everyone_mprs;
+  domain->mpr->_refcount++;
+}
+
+static void
+_cb_update_everyone_mpr(void) {
+  struct nhdp_neighbor *neigh;
+  struct nhdp_domain *domain;
+  struct nhdp_neighbor_domaindata *domaindata;
+
+  list_for_each_element(nhdp_db_get_neigh_list(), neigh, _global_node) {
+    if (_flooding_domain.mpr == &_everyone_mprs) {
+      neigh->neigh_is_flooding_mpr =
+          neigh->flooding_willingness > RFC7181_WILLINGNESS_NEVER;
+    }
+
+    list_for_each_element(nhdp_domain_get_list(), domain, _node) {
+      if (domain->mpr == &_everyone_mprs) {
+        domaindata = nhdp_domain_get_neighbordata(domain, neigh);
+        domaindata->neigh_is_mpr =
+            domaindata->willingness > RFC7181_WILLINGNESS_NEVER;
+      }
+    }
+  }
 }
 
 /**
