@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <errno.h>
 
 #include "common/avl.h"
@@ -83,7 +87,7 @@ static void _handle_nhdp_routes(struct nhdp_domain *);
 static void _add_route_to_kernel_queue(struct olsrv2_routing_entry *rtentry);
 static void _process_dijkstra_result(struct nhdp_domain *);
 static void _process_kernel_queue(void);
-static void _cb_trigger_dijkstra(void *);
+static void _cb_trigger_dijkstra(struct oonf_timer_instance *);
 static void _cb_nhdp_update(struct nhdp_neighbor *);
 static void _cb_route_finished(struct os_route *route, int error);
 
@@ -133,7 +137,7 @@ olsrv2_routing_init(void) {
   oonf_timer_add(&_dijkstra_timer_info);
 
   for (i=0; i<NHDP_MAXIMUM_DOMAINS; i++) {
-    avl_init(&_routing_tree[i], os_route_avl_cmp_route_key, false);
+    avl_init(&_routing_tree[i], os_routing_avl_cmp_route_key, false);
   }
   list_init_head(&_routing_filter_list);
   avl_init(&_dijkstra_working_tree, avl_comp_uint32, true);
@@ -345,11 +349,20 @@ olsrv2_routing_set_domain_parameter(struct nhdp_domain *domain,
   _trigger_dijkstra = true;
 }
 
+/**
+ * Get tree of olsrv2 routing entries
+ * @param domain nhdp domain
+ * @return tree of routing entries
+ */
 struct avl_tree *
 olsrv2_routing_get_tree(struct nhdp_domain *domain) {
   return &_routing_tree[domain->index];
 }
 
+/**
+ * Get list of olsrv2 routing filters
+ * @return filter list
+ */
 struct list_entity *
 olsrv2_routing_get_filter_list(void) {
   return &_routing_filter_list;
@@ -401,8 +414,8 @@ _add_entry(struct nhdp_domain *domain, struct os_route_key *prefix) {
   }
 
   /* set key */
-  memcpy(&rtentry->route.key, prefix, sizeof(struct os_route_key));
-  rtentry->_node.key = &rtentry->route.key;
+  memcpy(&rtentry->route.p.key, prefix, sizeof(struct os_route_key));
+  rtentry->_node.key = &rtentry->route.p.key;
 
   /* set domain */
   rtentry->domain = domain;
@@ -411,7 +424,7 @@ _add_entry(struct nhdp_domain *domain, struct os_route_key *prefix) {
   rtentry->path_cost = RFC7181_METRIC_INFINITE_PATH;
   rtentry->path_hops = 255;
   rtentry->route.cb_finished = _cb_route_finished;
-  rtentry->route.family = netaddr_get_address_family(&prefix->dst);
+  rtentry->route.p.family = netaddr_get_address_family(&prefix->dst);
 
   avl_insert(&_routing_tree[domain->index], &rtentry->_node);
   return rtentry;
@@ -571,15 +584,15 @@ _update_routing_entry(struct nhdp_domain *domain,
 
   neighdata = nhdp_domain_get_neighbordata(domain, first_hop);
   OONF_DEBUG(LOG_OLSRV2_ROUTING, "Initialize route entry dst %s [%s] with pathcost %u",
-      netaddr_to_string(&nbuf1, &rtentry->route.key.dst),
-      netaddr_to_string(&nbuf2, &rtentry->route.key.src),
+      netaddr_to_string(&nbuf1, &rtentry->route.p.key.dst),
+      netaddr_to_string(&nbuf2, &rtentry->route.p.key.src),
       pathcost);
 
   /* copy route parameters into data structure */
-  rtentry->route.if_index = neighdata->best_link_ifindex;
+  rtentry->route.p.if_index = neighdata->best_link_ifindex;
   rtentry->path_cost = pathcost;
   rtentry->path_hops = path_hops;
-  rtentry->route.metric = distance;
+  rtentry->route.p.metric = distance;
 
   /* remember next hop originator */
   memcpy(&rtentry->next_originator, &first_hop->originator, sizeof(struct netaddr));
@@ -593,11 +606,11 @@ _update_routing_entry(struct nhdp_domain *domain,
   /* copy gateway if necessary */
   if (single_hop
       && netaddr_cmp(&neighdata->best_link->if_addr,
-          &rtentry->route.key.dst) == 0) {
-    netaddr_invalidate(&rtentry->route.gw);
+          &rtentry->route.p.key.dst) == 0) {
+    netaddr_invalidate(&rtentry->route.p.gw);
   }
   else {
-    memcpy(&rtentry->route.gw, &neighdata->best_link->if_addr,
+    memcpy(&rtentry->route.p.gw, &neighdata->best_link->if_addr,
         sizeof(struct netaddr));
   }
 }
@@ -614,9 +627,7 @@ _prepare_routes(struct nhdp_domain *domain) {
   /* prepare all existing routing entries and put them into the working queue */
   avl_for_each_element(&_routing_tree[domain->index], rtentry, _node) {
     rtentry->set = false;
-    rtentry->_old_if_index = rtentry->route.if_index;
-    rtentry->_old_distance = rtentry->route.metric;
-    memcpy(&rtentry->_old_next_hop, &rtentry->route.gw, sizeof(struct netaddr));
+    memcpy(&rtentry->_old, &rtentry->route.p, sizeof(rtentry->_old));
   }
 }
 
@@ -645,7 +656,7 @@ _prepare_nodes(void) {
   avl_for_each_element(olsrv2_tc_get_endpoint_tree(), end, _node) {
     end->target._dijkstra.first_hop = NULL;
     end->target._dijkstra.path_cost = RFC7181_METRIC_INFINITE_PATH;
-    node->target._dijkstra.path_hops = 255;
+    end->target._dijkstra.path_hops = 255;
     end->target._dijkstra.done = false;
   }
 }
@@ -871,11 +882,17 @@ _handle_nhdp_routes(struct nhdp_domain *domain) {
         continue;
       }
 
-      os_route_init_sourcespec_prefix(&ssprefix, &naddr->neigh_addr);
+      os_routing_init_sourcespec_prefix(&ssprefix, &naddr->neigh_addr);
 
       /* update routing entry */
-      _update_routing_entry(domain, &ssprefix,
-          neigh, 0, neighcost, 1, true, olsrv2_originator_get(family));
+      if (olsrv2_originator_get(family)) {
+        _update_routing_entry(domain, &ssprefix,
+            neigh, 0, neighcost, 1, true, olsrv2_originator_get(family));
+      }
+      else {
+        _update_routing_entry(domain, &ssprefix,
+            neigh, 0, neighcost, 1, true, &NETADDR_UNSPEC);
+      }
     }
 
     list_for_each_element(&neigh->_links, lnk, _neigh_node) {
@@ -893,7 +910,7 @@ _handle_nhdp_routes(struct nhdp_domain *domain) {
 
         l2hop_pathcost += neighcost;
 
-        os_route_init_sourcespec_prefix(&ssprefix, &l2hop->twohop_addr);
+        os_routing_init_sourcespec_prefix(&ssprefix, &l2hop->twohop_addr);
 
         /* the 2-hop route is better than the dijkstra calculation */
         _update_routing_entry(domain, &ssprefix,
@@ -910,24 +927,16 @@ _handle_nhdp_routes(struct nhdp_domain *domain) {
 static void
 _add_route_to_kernel_queue(struct olsrv2_routing_entry *rtentry) {
 #ifdef OONF_LOG_INFO
-  struct os_route_str rbuf;
-  struct netaddr_str nbuf;
+  struct os_route_str rbuf1, rbuf2;
 #endif
 
   if (rtentry->set) {
     OONF_INFO(LOG_OLSRV2_ROUTING,
-        "Set route %s (%u %u %s)",
-        os_routing_to_string(&rbuf, &rtentry->route),
-        rtentry->_old_if_index, rtentry->_old_distance,
-        netaddr_to_string(&nbuf, &rtentry->_old_next_hop));
+        "Set route %s (%s)",
+        os_routing_to_string(&rbuf1, &rtentry->route.p),
+        os_routing_to_string(&rbuf2, &rtentry->_old));
 
-    if (_domain_parameter[rtentry->domain->index].use_srcip_in_routes
-        && netaddr_get_address_family(&rtentry->route.key.dst) == AF_INET) {
-      memcpy(&rtentry->route.src_ip, olsrv2_originator_get(AF_INET),
-          sizeof(rtentry->route.src_ip));
-    }
-
-    if (netaddr_get_address_family(&rtentry->route.gw) == AF_UNSPEC) {
+    if (netaddr_get_address_family(&rtentry->route.p.gw) == AF_UNSPEC) {
       /* insert/update single-hop routes early */
       list_add_head(&_kernel_queue, &rtentry->_working_node);
     }
@@ -939,10 +948,10 @@ _add_route_to_kernel_queue(struct olsrv2_routing_entry *rtentry) {
   else {
     OONF_INFO(LOG_OLSRV2_ROUTING,
         "Dijkstra result: remove route %s",
-        os_routing_to_string(&rbuf, &rtentry->route));
+        os_routing_to_string(&rbuf1, &rtentry->route.p));
 
 
-    if (netaddr_get_address_family(&rtentry->route.gw) == AF_UNSPEC) {
+    if (netaddr_get_address_family(&rtentry->route.p.gw) == AF_UNSPEC) {
       /* remove single-hop routes late */
       list_add_tail(&_kernel_queue, &rtentry->_working_node);
     }
@@ -965,21 +974,27 @@ _process_dijkstra_result(struct nhdp_domain *domain) {
 
   avl_for_each_element(&_routing_tree[domain->index], rtentry, _node) {
     /* initialize rest of route parameters */
-    rtentry->route.table = _domain_parameter[rtentry->domain->index].table;
-    rtentry->route.protocol = _domain_parameter[rtentry->domain->index].protocol;
-    rtentry->route.metric = _domain_parameter[rtentry->domain->index].distance;
+    rtentry->route.p.table = _domain_parameter[rtentry->domain->index].table;
+    rtentry->route.p.protocol = _domain_parameter[rtentry->domain->index].protocol;
+    rtentry->route.p.metric = _domain_parameter[rtentry->domain->index].distance;
+
+    if (rtentry->set
+        && _domain_parameter[rtentry->domain->index].use_srcip_in_routes
+        && netaddr_get_address_family(&rtentry->route.p.key.dst) == AF_INET) {
+      /* copy source address to route */
+      memcpy(&rtentry->route.p.src_ip, olsrv2_originator_get(AF_INET),
+          sizeof(rtentry->route.p.src_ip));
+    }
 
     list_for_each_element(&_routing_filter_list, filter, _node) {
-      if (!filter->filter(domain, &rtentry->route)) {
-        /* route was dropped by filter */
+      if (!filter->filter(domain, &rtentry->route.p, rtentry->set)) {
+        /* route modification was dropped by filter */
         continue;
       }
     }
 
     if (rtentry->set
-        && rtentry->_old_if_index == rtentry->route.if_index
-        && rtentry->_old_distance == rtentry->route.metric
-        && netaddr_cmp(&rtentry->_old_next_hop, &rtentry->route.gw) == 0) {
+        && memcmp(&rtentry->_old, &rtentry->route.p, sizeof(rtentry->_old)) == 0) {
       /* no change, ignore this entry */
       continue;
     }
@@ -1010,14 +1025,14 @@ _process_kernel_queue(void) {
       /* add to kernel */
       if (os_routing_set(&rtentry->route, true, true)) {
         OONF_WARN(LOG_OLSRV2_ROUTING, "Could not set route %s",
-            os_routing_to_string(&rbuf, &rtentry->route));
+            os_routing_to_string(&rbuf, &rtentry->route.p));
       }
     }
     else  {
       /* remove from kernel */
       if (os_routing_set(&rtentry->route, false, false)) {
         OONF_WARN(LOG_OLSRV2_ROUTING, "Could not remove route %s",
-            os_routing_to_string(&rbuf, &rtentry->route));
+            os_routing_to_string(&rbuf, &rtentry->route.p));
       }
     }
   }
@@ -1026,10 +1041,10 @@ _process_kernel_queue(void) {
 /**
  * Callback for checking if dijkstra was triggered during
  * rate limitation time
- * @param unused
+ * @param ptr timer instance that fired
  */
 static void
-_cb_trigger_dijkstra(void *unused __attribute__((unused))) {
+_cb_trigger_dijkstra(struct oonf_timer_instance *ptr __attribute__((unused))) {
   if (_trigger_dijkstra) {
     _trigger_dijkstra = false;
     olsrv2_routing_force_update(false);
@@ -1062,7 +1077,7 @@ _cb_route_finished(struct os_route *route, int error) {
 
   if (!rtentry->set && error == ESRCH) {
     OONF_DEBUG(LOG_OLSRV2_ROUTING, "Route %s was already gone",
-        os_routing_to_string(&rbuf, &rtentry->route));
+        os_routing_to_string(&rbuf, &rtentry->route.p));
   }
   else if (error) {
     if (error == -1) {
@@ -1072,7 +1087,7 @@ _cb_route_finished(struct os_route *route, int error) {
     /* an error happened, try again later */
     OONF_WARN(LOG_OLSRV2_ROUTING, "Error in route %s %s: %s (%d)",
         rtentry->set ? "setting" : "removal",
-            os_routing_to_string(&rbuf, &rtentry->route),
+            os_routing_to_string(&rbuf, &rtentry->route.p),
             strerror(error), error);
 
     if (error == EEXIST && rtentry->set) {
@@ -1092,11 +1107,11 @@ _cb_route_finished(struct os_route *route, int error) {
   if (rtentry->set) {
     /* route was set/updated successfully */
     OONF_INFO(LOG_OLSRV2_ROUTING, "Successfully set route %s",
-        os_routing_to_string(&rbuf, &rtentry->route));
+        os_routing_to_string(&rbuf, &rtentry->route.p));
   }
   else {
     OONF_INFO(LOG_OLSRV2_ROUTING, "Successfully removed route %s",
-        os_routing_to_string(&rbuf, &rtentry->route));
+        os_routing_to_string(&rbuf, &rtentry->route.p));
     _remove_entry(rtentry);
   }
 }
