@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 /* must be first because of a problem with linux/rtnetlink.h */
 #include <sys/socket.h>
 
@@ -99,7 +103,7 @@ static struct oonf_subsystem _oonf_os_routing_subsystem = {
 DECLARE_OONF_PLUGIN(_oonf_os_routing_subsystem);
 
 /* translation table between route types */
-struct route_type_translation _type_translation[] = {
+static struct route_type_translation _type_translation[] = {
     { OS_ROUTE_UNICAST, RTN_UNICAST },
     { OS_ROUTE_LOCAL, RTN_LOCAL },
     { OS_ROUTE_BROADCAST, RTN_BROADCAST },
@@ -116,7 +120,7 @@ static const uint32_t _rtnetlink_mcast[] = {
   RTNLGRP_IPV4_ROUTE, RTNLGRP_IPV6_ROUTE
 };
 
-struct os_system_netlink _rtnetlink_socket = {
+static struct os_system_netlink _rtnetlink_socket = {
   .name = "routing",
   .used_by = &_oonf_os_routing_subsystem,
   .cb_message = _cb_rtnetlink_message,
@@ -125,7 +129,7 @@ struct os_system_netlink _rtnetlink_socket = {
   .cb_timeout = _cb_rtnetlink_timeout,
 };
 
-struct os_system_netlink _rtnetlink_event_socket = {
+static struct os_system_netlink _rtnetlink_event_socket = {
   .name = "routing listener",
   .used_by = &_oonf_os_routing_subsystem,
   .cb_message = _cb_rtnetlink_event_message,
@@ -135,7 +139,7 @@ static struct avl_tree _rtnetlink_feedback;
 static struct list_entity _rtnetlink_listener;
 
 /* default wildcard route */
-static const struct os_route OS_ROUTE_WILDCARD = {
+static const struct os_route_parameter OS_ROUTE_WILDCARD = {
   .family = AF_UNSPEC,
   .src_ip = { ._type = AF_UNSPEC },
   .gw = { ._type = AF_UNSPEC },
@@ -159,24 +163,24 @@ static bool _is_kernel_3_11_0_or_better;
  */
 static int
 _init(void) {
-  if (os_system_netlink_add(&_rtnetlink_socket, NETLINK_ROUTE)) {
+  if (os_system_linux_netlink_add(&_rtnetlink_socket, NETLINK_ROUTE)) {
     return -1;
   }
 
-  if (os_system_netlink_add(&_rtnetlink_event_socket, NETLINK_ROUTE)) {
-    os_system_netlink_remove(&_rtnetlink_socket);
+  if (os_system_linux_netlink_add(&_rtnetlink_event_socket, NETLINK_ROUTE)) {
+    os_system_linux_netlink_remove(&_rtnetlink_socket);
     return -1;
   }
 
-  if (os_system_netlink_add_mc(&_rtnetlink_event_socket, _rtnetlink_mcast, ARRAYSIZE(_rtnetlink_mcast))) {
-    os_system_netlink_remove(&_rtnetlink_socket);
-    os_system_netlink_remove(&_rtnetlink_event_socket);
+  if (os_system_linux_netlink_add_mc(&_rtnetlink_event_socket, _rtnetlink_mcast, ARRAYSIZE(_rtnetlink_mcast))) {
+    os_system_linux_netlink_remove(&_rtnetlink_socket);
+    os_system_linux_netlink_remove(&_rtnetlink_event_socket);
     return -1;
   }
   avl_init(&_rtnetlink_feedback, avl_comp_uint32, false);
   list_init_head(&_rtnetlink_listener);
 
-  _is_kernel_3_11_0_or_better = os_linux_system_is_minimal_kernel(3,11,0);
+  _is_kernel_3_11_0_or_better = os_system_linux_is_minimal_kernel(3,11,0);
   return 0;
 }
 
@@ -191,12 +195,18 @@ _cleanup(void) {
     _routing_finished(rt, 1);
   }
 
-  os_system_netlink_remove(&_rtnetlink_socket);
-  os_system_netlink_remove(&_rtnetlink_event_socket);
+  os_system_linux_netlink_remove(&_rtnetlink_socket);
+  os_system_linux_netlink_remove(&_rtnetlink_event_socket);
 }
 
+/**
+ * Check if kernel supports source-specific routing
+ * @param af_family address family
+ * @return true if source-specific routing is supported for
+ *   address family
+ */
 bool
-os_routing_supports_source_specific(int af_family) {
+os_routing_linux_supports_source_specific(int af_family) {
   if (af_family == AF_INET) {
     return false;
   }
@@ -216,7 +226,7 @@ os_routing_supports_source_specific(int af_family) {
  * @return -1 if an error happened, 0 otherwise
  */
 int
-os_routing_set(struct os_route *route, bool set, bool del_similar) {
+os_routing_linux_set(struct os_route *route, bool set, bool del_similar) {
   uint8_t buffer[UIO_MAXIOV];
   struct nlmsghdr *msg;
   unsigned char scope;
@@ -248,33 +258,33 @@ os_routing_set(struct os_route *route, bool set, bool del_similar) {
   } else {
     msg->nlmsg_type = RTM_DELROUTE;
 
-    os_rt.protocol = 0;
-    netaddr_invalidate(&os_rt.src_ip);
+    os_rt.p.protocol = 0;
+    netaddr_invalidate(&os_rt.p.src_ip);
 
     if (del_similar) {
       /* no interface necessary */
-      os_rt.if_index = 0;
+      os_rt.p.if_index = 0;
 
       /* as wildcard for fuzzy deletion */
       scope = RT_SCOPE_NOWHERE;
     }
   }
 
-  if (netaddr_get_address_family(&os_rt.gw) == AF_UNSPEC
-      && netaddr_get_prefix_length(&os_rt.key.dst) == netaddr_get_maxprefix(&os_rt.key.dst)) {
+  if (netaddr_get_address_family(&os_rt.p.gw) == AF_UNSPEC
+      && netaddr_get_prefix_length(&os_rt.p.key.dst) == netaddr_get_maxprefix(&os_rt.p.key.dst)) {
     /* use destination as gateway, to 'force' linux kernel to do proper source address selection */
-    os_rt.gw = os_rt.key.dst;
+    os_rt.p.gw = os_rt.p.key.dst;
   }
 
   OONF_DEBUG(LOG_OS_ROUTING, "%sset route: %s", set ? "" : "re",
-      os_routing_to_string(&rbuf, &os_rt));
+      os_routing_to_string(&rbuf, &os_rt.p));
 
   if (_routing_set(msg, &os_rt, scope)) {
     return -1;
   }
 
   /* cannot fail */
-  seq = os_system_netlink_send(&_rtnetlink_socket, msg);
+  seq = os_system_linux_netlink_send(&_rtnetlink_socket, msg);
 
   if (route->cb_finished) {
     route->_internal.nl_seq = seq;
@@ -292,7 +302,7 @@ os_routing_set(struct os_route *route, bool set, bool del_similar) {
  * @return -1 if an error happened, 0 otherwise
  */
 int
-os_routing_query(struct os_route *route) {
+os_routing_linux_query(struct os_route *route) {
   uint8_t buffer[UIO_MAXIOV];
   struct nlmsghdr *msg;
   struct rtgenmsg *rt_gen;
@@ -311,9 +321,9 @@ os_routing_query(struct os_route *route) {
   msg->nlmsg_len = NLMSG_LENGTH(sizeof(*rt_gen));
 
   msg->nlmsg_type = RTM_GETROUTE;
-  rt_gen->rtgen_family = route->family;
+  rt_gen->rtgen_family = route->p.family;
 
-  seq = os_system_netlink_send(&_rtnetlink_socket, msg);
+  seq = os_system_linux_netlink_send(&_rtnetlink_socket, msg);
   if (seq < 0) {
     return -1;
   }
@@ -329,8 +339,8 @@ os_routing_query(struct os_route *route) {
  * @param route pointer to os_route
  */
 void
-os_routing_interrupt(struct os_route *route) {
-  if (os_routing_is_in_progress(route)) {
+os_routing_linux_interrupt(struct os_route *route) {
+  if (os_routing_linux_is_in_progress(route)) {
     _routing_finished(route, -1);
   }
 }
@@ -341,25 +351,36 @@ os_routing_interrupt(struct os_route *route) {
  *   false otherwise
  */
 bool
-os_routing_is_in_progress(struct os_route *route) {
+os_routing_linux_is_in_progress(struct os_route *route) {
   return avl_is_node_added(&route->_internal._node);
 }
 
-const struct os_route *
-os_routing_get_wildcard_route(void) {
-  return &OS_ROUTE_WILDCARD;
-}
-
+/**
+ * Add routing change listener
+ * @param listener routing change listener
+ */
 void
-os_routing_listener_add(struct os_route_listener *listener) {
+os_routing_linux_listener_add(struct os_route_listener *listener) {
   list_add_tail(&_rtnetlink_listener, &listener->_internal._node);
 }
 
+/**
+ * Remove routing change listener
+ * @param listener routing change listener
+ */
 void
-os_routing_listener_remove(struct os_route_listener *listener) {
+os_routing_linux_listener_remove(struct os_route_listener *listener) {
   list_remove(&listener->_internal._node);
 }
 
+/**
+ * @return wildcard route
+ */
+const struct os_route_parameter *
+os_routing_linux_get_wildcard_route(void) {
+  return &OS_ROUTE_WILDCARD;
+}
+
 /**
  * Stop processing of a routing command and set error code
  * for callback
@@ -390,98 +411,98 @@ _routing_set(struct nlmsghdr *msg, struct os_route *route,
   size_t i;
 
   /* calculate address af_type */
-  if (netaddr_get_address_family(&route->key.dst) != AF_UNSPEC) {
-    route->family = netaddr_get_address_family(&route->key.dst);
+  if (netaddr_get_address_family(&route->p.key.dst) != AF_UNSPEC) {
+    route->p.family = netaddr_get_address_family(&route->p.key.dst);
   }
-  if (netaddr_get_address_family(&route->gw) != AF_UNSPEC) {
-    if (route->family  != AF_UNSPEC
-        && route->family  != netaddr_get_address_family(&route->gw)) {
+  if (netaddr_get_address_family(&route->p.gw) != AF_UNSPEC) {
+    if (route->p.family  != AF_UNSPEC
+        && route->p.family  != netaddr_get_address_family(&route->p.gw)) {
       return -1;
     }
-    route->family  = netaddr_get_address_family(&route->gw);
+    route->p.family  = netaddr_get_address_family(&route->p.gw);
   }
-  if (netaddr_get_address_family(&route->src_ip) != AF_UNSPEC) {
-    if (route->family  != AF_UNSPEC && route->family  != netaddr_get_address_family(&route->src_ip)) {
+  if (netaddr_get_address_family(&route->p.src_ip) != AF_UNSPEC) {
+    if (route->p.family  != AF_UNSPEC && route->p.family  != netaddr_get_address_family(&route->p.src_ip)) {
       return -1;
     }
-    route->family  = netaddr_get_address_family(&route->src_ip);
+    route->p.family  = netaddr_get_address_family(&route->p.src_ip);
   }
 
-  if (route->family  == AF_UNSPEC) {
-    route->family  = AF_INET;
+  if (route->p.family  == AF_UNSPEC) {
+    route->p.family  = AF_INET;
   }
 
   /* initialize rtmsg payload */
   rt_msg = NLMSG_DATA(msg);
 
-  rt_msg->rtm_family = route->family ;
+  rt_msg->rtm_family = route->p.family ;
   rt_msg->rtm_scope = rt_scope;
-  rt_msg->rtm_protocol = route->protocol;
-  rt_msg->rtm_table = route->table;
+  rt_msg->rtm_protocol = route->p.protocol;
+  rt_msg->rtm_table = route->p.table;
 
   /* set default route type */
   rt_msg->rtm_type = RTN_UNICAST;
 
   /* set route type */
   for (i=0; i<ARRAYSIZE(_type_translation); i++) {
-    if (_type_translation[i].oonf == route->type) {
+    if (_type_translation[i].oonf == route->p.type) {
       rt_msg->rtm_type = _type_translation[i].os_linux;
       break;
     }
   }
 
   /* add attributes */
-  if (netaddr_get_address_family(&route->src_ip) != AF_UNSPEC) {
+  if (netaddr_get_address_family(&route->p.src_ip) != AF_UNSPEC) {
     /* add src-ip */
-    if (os_system_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_PREFSRC, &route->src_ip)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
+        msg, RTA_PREFSRC, &route->p.src_ip)) {
       return -1;
     }
   }
 
-  if (netaddr_get_address_family(&route->gw) != AF_UNSPEC) {
+  if (netaddr_get_address_family(&route->p.gw) != AF_UNSPEC) {
     rt_msg->rtm_flags |= RTNH_F_ONLINK;
 
     /* add gateway */
-    if (os_system_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_GATEWAY, &route->gw)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
+        msg, RTA_GATEWAY, &route->p.gw)) {
       return -1;
     }
   }
 
-  if (netaddr_get_address_family(&route->key.dst) != AF_UNSPEC) {
-    rt_msg->rtm_dst_len = netaddr_get_prefix_length(&route->key.dst);
+  if (netaddr_get_address_family(&route->p.key.dst) != AF_UNSPEC) {
+    rt_msg->rtm_dst_len = netaddr_get_prefix_length(&route->p.key.dst);
 
     /* add destination */
-    if (os_system_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_DST, &route->key.dst)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
+        msg, RTA_DST, &route->p.key.dst)) {
       return -1;
     }
   }
 
-  if (netaddr_get_address_family(&route->key.src) == AF_INET6
-      && netaddr_get_prefix_length(&route->key.src) != 0) {
-    rt_msg->rtm_src_len = netaddr_get_prefix_length(&route->key.src);
+  if (netaddr_get_address_family(&route->p.key.src) == AF_INET6
+      && netaddr_get_prefix_length(&route->p.key.src) != 0) {
+    rt_msg->rtm_src_len = netaddr_get_prefix_length(&route->p.key.src);
 
     /* add source-specific routing prefix */
-    if (os_system_netlink_addnetaddr(&_rtnetlink_event_socket,
-        msg, RTA_SRC, &route->key.src)) {
+    if (os_system_linux_netlink_addnetaddr(&_rtnetlink_event_socket,
+        msg, RTA_SRC, &route->p.key.src)) {
       return -1;
     }
   }
 
-  if (route->metric != -1) {
+  if (route->p.metric != -1) {
     /* add metric */
-    if (os_system_netlink_addreq(&_rtnetlink_event_socket,
-        msg, RTA_PRIORITY, &route->metric, sizeof(route->metric))) {
+    if (os_system_linux_netlink_addreq(&_rtnetlink_event_socket,
+        msg, RTA_PRIORITY, &route->p.metric, sizeof(route->p.metric))) {
       return -1;
     }
   }
 
-  if (route->if_index) {
+  if (route->p.if_index) {
     /* add interface*/
-    if (os_system_netlink_addreq(&_rtnetlink_event_socket,
-        msg, RTA_OIF, &route->if_index, sizeof(route->if_index))) {
+    if (os_system_linux_netlink_addreq(&_rtnetlink_event_socket,
+        msg, RTA_OIF, &route->p.if_index, sizeof(route->p.if_index))) {
       return -1;
     }
   }
@@ -511,25 +532,25 @@ _routing_parse_nlmsg(struct os_route *route, struct nlmsghdr *msg) {
     return 1;
   }
 
-  memcpy(route, &OS_ROUTE_WILDCARD, sizeof(*route));
+  memcpy(&route->p, &OS_ROUTE_WILDCARD, sizeof(OS_ROUTE_WILDCARD));
 
-  route->protocol = rt_msg->rtm_protocol;
-  route->table = rt_msg->rtm_table;
-  route->family = rt_msg->rtm_family;
+  route->p.protocol = rt_msg->rtm_protocol;
+  route->p.table = rt_msg->rtm_table;
+  route->p.family = rt_msg->rtm_family;
 
-  if (route->family != AF_INET && route->family != AF_INET6) {
+  if (route->p.family != AF_INET && route->p.family != AF_INET6) {
     return -1;
   }
 
   /* get route type */
-  route->type = OS_ROUTE_UNDEFINED;
+  route->p.type = OS_ROUTE_UNDEFINED;
   for (i=0; i<ARRAYSIZE(_type_translation); i++) {
     if (rt_msg->rtm_type == _type_translation[i].os_linux) {
-      route->type = _type_translation[i].oonf;
+      route->p.type = _type_translation[i].oonf;
       break;
     }
   }
-  if (route->type == OS_ROUTE_UNDEFINED) {
+  if (route->p.type == OS_ROUTE_UNDEFINED) {
     OONF_WARN(LOG_OS_ROUTING, "Got route type: %u", rt_msg->rtm_type);
     return -1;
   }
@@ -537,35 +558,35 @@ _routing_parse_nlmsg(struct os_route *route, struct nlmsghdr *msg) {
   for(; RTA_OK(rt_attr, rt_len); rt_attr = RTA_NEXT(rt_attr,rt_len)) {
     switch(rt_attr->rta_type) {
       case RTA_PREFSRC:
-        netaddr_from_binary(&route->src_ip, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr),
+        netaddr_from_binary(&route->p.src_ip, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr),
             rt_msg->rtm_family);
         break;
       case RTA_GATEWAY:
-        netaddr_from_binary(&route->gw, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr), rt_msg->rtm_family);
+        netaddr_from_binary(&route->p.gw, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr), rt_msg->rtm_family);
         break;
       case RTA_DST:
-        netaddr_from_binary_prefix(&route->key.dst, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr),
+        netaddr_from_binary_prefix(&route->p.key.dst, RTA_DATA(rt_attr), RTA_PAYLOAD(rt_attr),
             rt_msg->rtm_family, rt_msg->rtm_dst_len);
         break;
       case RTA_SRC:
-        netaddr_from_binary_prefix(&route->key.src, RTA_DATA(rt_attr),
+        netaddr_from_binary_prefix(&route->p.key.src, RTA_DATA(rt_attr),
             RTA_PAYLOAD(rt_attr), rt_msg->rtm_family, rt_msg->rtm_src_len);
         break;
       case RTA_PRIORITY:
-        memcpy(&route->metric, RTA_DATA(rt_attr), sizeof(route->metric));
+        memcpy(&route->p.metric, RTA_DATA(rt_attr), sizeof(route->p.metric));
         break;
       case RTA_OIF:
-        memcpy(&route->if_index, RTA_DATA(rt_attr), sizeof(route->if_index));
+        memcpy(&route->p.if_index, RTA_DATA(rt_attr), sizeof(route->p.if_index));
         break;
       default:
         break;
     }
   }
 
-  if (netaddr_get_address_family(&route->key.dst) == AF_UNSPEC) {
-    memcpy(&route->key.dst, route->family == AF_INET ? &NETADDR_IPV4_ANY : &NETADDR_IPV6_ANY,
-        sizeof(route->key.dst));
-    netaddr_set_prefix_length(&route->key.dst, rt_msg->rtm_dst_len);
+  if (netaddr_get_address_family(&route->p.key.dst) == AF_UNSPEC) {
+    memcpy(&route->p.key.dst, route->p.family == AF_INET ? &NETADDR_IPV4_ANY : &NETADDR_IPV6_ANY,
+        sizeof(route->p.key.dst));
+    netaddr_set_prefix_length(&route->p.key.dst, rt_msg->rtm_dst_len);
   }
   return 0;
 }
@@ -578,38 +599,38 @@ _routing_parse_nlmsg(struct os_route *route, struct nlmsghdr *msg) {
  */
 static bool
 _match_routes(struct os_route *filter, struct os_route *route) {
-  if (filter->family != AF_UNSPEC && filter->family != route->family) {
+  if (filter->p.family != AF_UNSPEC && filter->p.family != route->p.family) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->src_ip) != AF_UNSPEC
-      && memcmp(&filter->src_ip, &route->src_ip, sizeof(filter->src_ip)) != 0) {
+  if (netaddr_get_address_family(&filter->p.src_ip) != AF_UNSPEC
+      && memcmp(&filter->p.src_ip, &route->p.src_ip, sizeof(filter->p.src_ip)) != 0) {
     return false;
   }
-  if (filter->type != OS_ROUTE_UNDEFINED && filter->type != route->type) {
+  if (filter->p.type != OS_ROUTE_UNDEFINED && filter->p.type != route->p.type) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->gw) != AF_UNSPEC
-      && memcmp(&filter->gw, &route->gw, sizeof(filter->gw)) != 0) {
+  if (netaddr_get_address_family(&filter->p.gw) != AF_UNSPEC
+      && memcmp(&filter->p.gw, &route->p.gw, sizeof(filter->p.gw)) != 0) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->key.dst) != AF_UNSPEC
-      && memcmp(&filter->key.dst, &route->key.dst, sizeof(filter->key.dst)) != 0) {
+  if (netaddr_get_address_family(&filter->p.key.dst) != AF_UNSPEC
+      && memcmp(&filter->p.key.dst, &route->p.key.dst, sizeof(filter->p.key.dst)) != 0) {
     return false;
   }
-  if (netaddr_get_address_family(&filter->key.src) != AF_UNSPEC
-      && memcmp(&filter->key.src, &route->key.src, sizeof(filter->key.src)) != 0) {
+  if (netaddr_get_address_family(&filter->p.key.src) != AF_UNSPEC
+      && memcmp(&filter->p.key.src, &route->p.key.src, sizeof(filter->p.key.src)) != 0) {
     return false;
   }
-  if (filter->metric != -1 && filter->metric != route->metric) {
+  if (filter->p.metric != -1 && filter->p.metric != route->p.metric) {
     return false;
   }
-  if (filter->table != RT_TABLE_UNSPEC && filter->table != route->table) {
+  if (filter->p.table != RT_TABLE_UNSPEC && filter->p.table != route->p.table) {
     return false;
   }
-  if (filter->protocol != RTPROT_UNSPEC && filter->protocol != route->protocol) {
+  if (filter->p.protocol != RTPROT_UNSPEC && filter->p.protocol != route->p.protocol) {
     return false;
   }
-  return filter->if_index == 0 || filter->if_index == route->if_index;
+  return filter->p.if_index == 0 || filter->p.if_index == route->p.if_index;
 }
 
 /**
@@ -690,7 +711,7 @@ _cb_rtnetlink_error(uint32_t seq, int err) {
   if (route) {
     OONF_DEBUG(LOG_OS_ROUTING, "Route seqno %u failed: %s (%d) %s",
         seq, strerror(err), err,
-        os_routing_to_string(&rbuf, route));
+        os_routing_to_string(&rbuf, &route->p));
 
     _routing_finished(route, err);
   }
@@ -730,7 +751,7 @@ _cb_rtnetlink_done(uint32_t seq) {
   route = avl_find_element(&_rtnetlink_feedback, &seq, route, _internal._node);
   if (route) {
     OONF_DEBUG(LOG_OS_ROUTING, "Route %s with seqno %u done",
-        os_routing_to_string(&rbuf, route), seq);
+        os_routing_to_string(&rbuf, &route->p), seq);
     _routing_finished(route, 0);
   }
 }
